@@ -21,22 +21,39 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
 import java.io.File;
+import java.io.FileFilter;
 import java.io.FilenameFilter;
 import java.io.IOException;
 import java.io.InputStream;
+import java.util.Random;
 
-import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
+import org.apache.poi.poifs.filesystem.POIFSFileSystem;
+import org.apache.tika.MultiThreadedTikaTest;
+import org.apache.tika.Tika;
+import org.apache.tika.config.TikaConfig;
+import org.apache.tika.exception.TikaException;
 import org.apache.tika.io.TikaInputStream;
 import org.apache.tika.metadata.Metadata;
 import org.apache.tika.mime.MediaType;
+import org.apache.tika.mime.MimeTypes;
+import org.apache.tika.parser.iwork.iwana.IWork13PackageParser;
+import org.apache.tika.utils.XMLReaderUtils;
+import org.junit.After;
 import org.junit.Test;
 
 /**
- * Junit test class for {@link ContainerAwareDetector}
+ * Junit test class for {@link org.apache.tika.parser.microsoft.POIFSContainerDetector}
  */
-public class TestContainerAwareDetector {
+public class TestContainerAwareDetector extends MultiThreadedTikaTest {
+    private final TikaConfig tikaConfig = TikaConfig.getDefaultConfig();
+    private final MimeTypes mimeTypes = tikaConfig.getMimeRepository();
+    private final Detector detector = new DefaultDetector(mimeTypes);
 
-    private final Detector detector = new DefaultDetector();
+    @After
+    public void tearDown() throws TikaException {
+        //make sure to reset pool size because it is being randomly resized during the tests
+        XMLReaderUtils.setPoolSize(10);
+    }
 
     private void assertTypeByData(String file, String type) throws Exception {
        assertTypeByNameAndData(file, null, type);
@@ -49,20 +66,27 @@ public class TestContainerAwareDetector {
        assertTypeByNameAndData(file, byNameAndData);
     }
     private void assertTypeByNameAndData(String dataFile, String name, String type) throws Exception {
-       TikaInputStream stream = TikaInputStream.get(
-               TestContainerAwareDetector.class.getResource(
-                       "/test-documents/" + dataFile));
-       try {
-           Metadata m = new Metadata();
-           if (name != null)
-              m.add(Metadata.RESOURCE_NAME_KEY, name);
-           
-           assertEquals(
-                   MediaType.parse(type),
-                   detector.detect(stream, m));
-       } finally {
-           stream.close();
-       }
+        assertTypeByNameAndData(dataFile, name, type, null);
+    }
+    private void assertTypeByNameAndData(String dataFile, String name, String typeFromDetector, String typeFromMagic) throws Exception {
+        try (TikaInputStream stream = TikaInputStream.get(
+                TestContainerAwareDetector.class.getResource("/test-documents/" + dataFile))) {
+            Metadata m = new Metadata();
+            if (name != null)
+                m.add(Metadata.RESOURCE_NAME_KEY, name);
+
+            // Mime Magic version is likely to be less precise
+            if (typeFromMagic != null) {
+                assertEquals(
+                        MediaType.parse(typeFromMagic),
+                        mimeTypes.detect(stream, m));
+            }
+
+            // All being well, the detector should get it perfect
+            assertEquals(
+                    MediaType.parse(typeFromDetector),
+                    detector.detect(stream, m));
+        }
     }
 
     @Test
@@ -78,6 +102,7 @@ public class TestContainerAwareDetector {
         assertTypeByData("testPUBLISHER.pub", "application/x-mspublisher");
         assertTypeByData("testWORKS.wps", "application/vnd.ms-works");
         assertTypeByData("testWORKS2000.wps", "application/vnd.ms-works");
+        
         // older Works Word Processor files can't be recognized
         // they were created with Works Word Processor 7.0 (hence the text inside)
         // and exported to the older formats with the "Save As" feature
@@ -86,13 +111,16 @@ public class TestContainerAwareDetector {
         assertTypeByData("testWORKSSpreadsheet7.0.xlr", "application/x-tika-msworks-spreadsheet");
         assertTypeByData("testPROJECT2003.mpp", "application/vnd.ms-project");
         assertTypeByData("testPROJECT2007.mpp", "application/vnd.ms-project");
+        
         // Excel95 can be detected by not parsed
         assertTypeByData("testEXCEL_95.xls", "application/vnd.ms-excel");
 
         // Try some ones that POI doesn't handle, that are still OLE2 based
         assertTypeByData("testCOREL.shw", "application/x-corelpresentations");
-        assertTypeByData("testQUATTRO.qpw", "application/x-quattro-pro");
-        assertTypeByData("testQUATTRO.wb3", "application/x-quattro-pro");
+        assertTypeByData("testQUATTRO.qpw", "application/x-quattro-pro; version=9");
+        assertTypeByData("testQUATTRO.wb3", "application/x-quattro-pro; version=7-8");
+        
+        assertTypeByData("testHWP_5.0.hwp", "application/x-hwp-v5");
         
         
         // With the filename and data
@@ -149,17 +177,13 @@ public class TestContainerAwareDetector {
 
     @Test
     public void testOpenContainer() throws Exception {
-        TikaInputStream stream = TikaInputStream.get(
-                TestContainerAwareDetector.class.getResource(
-                        "/test-documents/testPPT.ppt"));
-        try {
+        try (TikaInputStream stream = TikaInputStream.get(
+                TestContainerAwareDetector.class.getResource("/test-documents/testPPT.ppt"))) {
             assertNull(stream.getOpenContainer());
             assertEquals(
                     MediaType.parse("application/vnd.ms-powerpoint"),
                     detector.detect(stream, new Metadata()));
-            assertTrue(stream.getOpenContainer() instanceof NPOIFSFileSystem);
-        } finally {
-            stream.close();
+            assertTrue(stream.getOpenContainer() instanceof POIFSFileSystem);
         }
     }
 
@@ -197,7 +221,16 @@ public class TestContainerAwareDetector {
         assertTypeByData("testPPT.ppsx", "application/vnd.openxmlformats-officedocument.presentationml.slideshow");
         assertTypeByData("testPPT.ppsm", "application/vnd.ms-powerpoint.slideshow.macroEnabled.12");
         assertTypeByData("testDOTM.dotm", "application/vnd.ms-word.template.macroEnabled.12");
+        assertTypeByData("testEXCEL.strict.xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
+        assertTypeByData("testPPT.xps", "application/vnd.ms-xpsdocument");
 
+        assertTypeByData("testVISIO.vsdm", "application/vnd.ms-visio.drawing.macroenabled.12");
+        assertTypeByData("testVISIO.vsdx", "application/vnd.ms-visio.drawing");
+        assertTypeByData("testVISIO.vssm", "application/vnd.ms-visio.stencil.macroenabled.12");
+        assertTypeByData("testVISIO.vssx", "application/vnd.ms-visio.stencil");
+        assertTypeByData("testVISIO.vstm", "application/vnd.ms-visio.template.macroenabled.12");
+        assertTypeByData("testVISIO.vstx", "application/vnd.ms-visio.template");
+        
         // .xlsb is an OOXML file containing the binary parts, and not
         //  an OLE2 file as you might initially expect!
         assertTypeByData("testEXCEL.xlsb", "application/vnd.ms-excel.sheet.binary.macroEnabled.12");
@@ -281,13 +314,9 @@ public class TestContainerAwareDetector {
     private void assertRemovalTempfiles(String fileName) throws Exception {
         int numberOfTempFiles = countTemporaryFiles();
 
-        TikaInputStream stream = TikaInputStream.get(
-                TestContainerAwareDetector.class.getResource(
-                        "/test-documents/" + fileName));
-        try {
+        try (TikaInputStream stream = TikaInputStream.get(
+                TestContainerAwareDetector.class.getResource("/test-documents/" + fileName))) {
             detector.detect(stream, new Metadata());
-        } finally {
-            stream.close();
         }
 
         assertEquals(numberOfTempFiles, countTemporaryFiles());
@@ -300,6 +329,18 @@ public class TestContainerAwareDetector {
         assertTypeByData("testPages.pages", "application/vnd.apple.pages");
     }
 
+    @Test
+    public void testDetectIWork2013() throws Exception {
+        assertTypeByData("testKeynote2013.key",
+                IWork13PackageParser.IWork13DocumentType.KEYNOTE13.getType().toString());
+        // Without decoding the Document snappy stream, we can't tell the
+        //  difference between these two just based on the zip entries
+        assertTypeByData("testNumbers2013.numbers",
+                IWork13PackageParser.IWork13DocumentType.UNKNOWN13.getType().toString());
+        assertTypeByData("testPages2013.pages",
+                IWork13PackageParser.IWork13DocumentType.UNKNOWN13.getType().toString());
+    }
+
     @Test
     public void testDetectKMZ() throws Exception {
        assertTypeByData("testKMZ.kmz", "application/vnd.google-earth.kmz");
@@ -309,7 +350,15 @@ public class TestContainerAwareDetector {
     public void testDetectIPA() throws Exception {
         assertTypeByNameAndData("testIPA.ipa", "application/x-itunes-ipa");
         assertTypeByData("testIPA.ipa", "application/x-itunes-ipa");
-     }
+    }
+    
+    @Test
+    public void testASiC() throws Exception {
+        assertTypeByData("testASiCE.asice", "application/vnd.etsi.asic-e+zip");
+        assertTypeByData("testASiCS.asics", "application/vnd.etsi.asic-s+zip");
+        assertTypeByNameAndData("testASiCE.asice", "application/vnd.etsi.asic-e+zip");
+        assertTypeByNameAndData("testASiCS.asics", "application/vnd.etsi.asic-s+zip");
+    }
      
     @Test
     public void testDetectZip() throws Exception {
@@ -321,14 +370,31 @@ public class TestContainerAwareDetector {
         assertTypeByData("testWAR.war", "application/x-tika-java-web-archive");
         assertTypeByData("testEAR.ear", "application/x-tika-java-enterprise-archive");
         assertTypeByData("testAPK.apk", "application/vnd.android.package-archive");
+        
+        // JAR with HTML files in it
+        assertTypeByNameAndData("testJAR_with_HTML.jar", "testJAR_with_HTML.jar",
+                                "application/java-archive", "application/java-archive");
+    }
+
+    @Test
+    public void testTarWithNoMagic() throws Exception {
+        assertTypeByData("testTAR_no_magic.tar", "application/x-tar");
+    }
+
+    @Test
+    public void testLZMAOOM() throws Exception {
+        assertTypeByData("testLZMA_oom", "application/x-lzma");
+    }
+
+    @Test
+    public void testCompressOOM() throws Exception {
+        assertTypeByData("testZ_oom.Z", "application/x-compress");
     }
 
     private TikaInputStream getTruncatedFile(String name, int n)
             throws IOException {
-        InputStream input =
-            TestContainerAwareDetector.class.getResourceAsStream(
-                    "/test-documents/" + name);
-        try {
+        try (InputStream input = TestContainerAwareDetector.class.getResourceAsStream(
+                "/test-documents/" + name)) {
             byte[] bytes = new byte[n];
             int m = 0;
             while (m < bytes.length) {
@@ -340,8 +406,6 @@ public class TestContainerAwareDetector {
                 }
             }
             return TikaInputStream.get(bytes);
-        } finally {
-            input.close();
         }
     }
 
@@ -350,51 +414,83 @@ public class TestContainerAwareDetector {
         // First up a truncated OOXML (zip) file
        
         // With only the data supplied, the best we can do is the container
-        TikaInputStream xlsx = getTruncatedFile("testEXCEL.xlsx", 300);
         Metadata m = new Metadata();
-        try {
+        try (TikaInputStream xlsx = getTruncatedFile("testEXCEL.xlsx", 300)) {
             assertEquals(
                     MediaType.application("x-tika-ooxml"),
                     detector.detect(xlsx, m));
-        } finally {
-            xlsx.close();
         }
         
         // With truncated data + filename, we can use the filename to specialise
-        xlsx = getTruncatedFile("testEXCEL.xlsx", 300);
         m = new Metadata();
         m.add(Metadata.RESOURCE_NAME_KEY, "testEXCEL.xlsx");
-        try {
+        try (TikaInputStream xlsx = getTruncatedFile("testEXCEL.xlsx", 300)) {
             assertEquals(
                     MediaType.application("vnd.openxmlformats-officedocument.spreadsheetml.sheet"),
                     detector.detect(xlsx, m));
-        } finally {
-            xlsx.close();
         }
-        
 
         // Now a truncated OLE2 file 
-        TikaInputStream xls = getTruncatedFile("testEXCEL.xls", 400);
         m = new Metadata();
-        try {
+        try (TikaInputStream xls = getTruncatedFile("testEXCEL.xls", 400)) {
             assertEquals(
                     MediaType.application("x-tika-msoffice"),
                     detector.detect(xls, m));
-        } finally {
-            xls.close();
         }
         
         // Finally a truncated OLE2 file, with a filename available
-        xls = getTruncatedFile("testEXCEL.xls", 400);
         m = new Metadata();
         m.add(Metadata.RESOURCE_NAME_KEY, "testEXCEL.xls");
-        try {
+        try (TikaInputStream xls = getTruncatedFile("testEXCEL.xls", 400)) {
             assertEquals(
                     MediaType.application("vnd.ms-excel"),
                     detector.detect(xls, m));
-        } finally {
-            xls.close();
         }
    }
 
+    @Test
+    public void testXMLMultiThreaded() throws Exception {
+        Detector detector = new Tika().getDetector();
+        FileFilter filter = new FileFilter() {
+            @Override
+            public boolean accept(File pathname) {
+                if (pathname.getName().endsWith(".xml")) {
+                    return true;
+                }
+                return false;
+            }
+        };
+        int numThreads = 1;
+        XMLReaderUtils.setPoolSize(numThreads);
+        testDetector(detector, numThreads, 20, filter, numThreads * 2);
+    }
+
+    @Test
+    public void testAllMultithreaded() throws Exception {
+
+        Detector detector = new Tika().getDetector();
+        FileFilter filter = new FileFilter() {
+            //TODO: create proper randomized framework that will record seed, etc...
+            private final Random random = new Random();
+            //increase this to the number of files for a true smoke test
+            //for now, randomly pick 20 files.
+            int toProcess = 20;
+            int processed = 0;
+            @Override
+            public boolean accept(File pathname) {
+                if (processed >= toProcess) {
+                    return false;
+                } else if (random.nextBoolean()) {
+                    processed++;
+                    return true;
+                }
+                return false;
+            }
+        };
+        int numThreads = 20;
+        XMLReaderUtils.setPoolSize(numThreads);
+
+        testDetector(detector, numThreads, 50, filter, numThreads*3);
+    }
+
 }
