@@ -16,6 +16,7 @@
  */
 package org.apache.tika.embedder;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
@@ -38,7 +39,6 @@ import java.util.Locale;
 import java.util.Map;
 
 import org.apache.tika.exception.TikaException;
-import org.apache.tika.io.IOUtils;
 import org.apache.tika.io.TemporaryResources;
 import org.apache.tika.io.TikaInputStream;
 import org.apache.tika.metadata.Metadata;
@@ -59,7 +59,7 @@ public class ExternalEmbedderTest {
 
     protected static final DateFormat EXPECTED_METADATA_DATE_FORMATTER =
             new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.ROOT);
-    protected static final String DEFAULT_CHARSET = IOUtils.UTF_8.name();
+    protected static final String DEFAULT_CHARSET = UTF_8.name();
     private static final String COMMAND_METADATA_ARGUMENT_DESCRIPTION = "dc:description";
     private static final String TEST_TXT_PATH = "/test-documents/testTXT.txt";
 
