@@ -769,6 +769,18 @@ public class OOXMLParserTest extends TikaTest {
         assertContains("\uD800\uDF32\uD800\uDF3f\uD800\uDF44\uD800\uDF39\uD800\uDF43\uD800\uDF3A", xml);
     }
 
+    @Test
+    public void testSkipHeaderFooter() throws Exception {
+        //now test turning off header/footer
+        OfficeParserConfig config = new OfficeParserConfig();
+        config.setIncludeHeadersAndFooters(false);
+        ParseContext context = new ParseContext();
+        context.set(OfficeParserConfig.class, config);
+        String xml = getXML("testPPT_various.pptx", context).xml;
+        assertNotContained("This is the header text", xml);
+
+    }
+
     @Test
     public void testCommentPPTX() throws Exception {
         XMLResult r = getXML("testPPT_comment.pptx");
@@ -789,6 +801,21 @@ public class OOXMLParserTest extends TikaTest {
         assertContains("Master footer is here", content);
     }
 
+    @Test
+    @Ignore("can't tell why this isn't working")
+    public void testTurningOffMasterContent() throws Exception {
+        //now test turning off master content
+
+        //the underlying xml has "Master footer" in
+        //the actual slide's xml, not just in the master slide.
+        OfficeParserConfig config = new OfficeParserConfig();
+        config.setIncludeSlideMasterContent(false);
+        ParseContext context = new ParseContext();
+        context.set(OfficeParserConfig.class, config);
+        String xml = getXML("testPPT_masterFooter.pptx", context).xml;
+        assertNotContained("Master footer", xml);
+    }
+
     /**
      * TIKA-712 Master Slide Text from PPT and PPTX files
      * should be extracted too
@@ -805,6 +832,14 @@ public class OOXMLParserTest extends TikaTest {
 
         String content = handler.toString();
         assertContains("Text that I added to the master slide", content);
+
+        //now test turning off master content
+        OfficeParserConfig config = new OfficeParserConfig();
+        config.setIncludeSlideMasterContent(false);
+        ParseContext context = new ParseContext();
+        context.set(OfficeParserConfig.class, config);
+        content = getXML("testPPT_masterText.pptx", context).xml;
+        assertNotContained("Text that I added", content);
     }
 
     @Test
@@ -819,6 +854,14 @@ public class OOXMLParserTest extends TikaTest {
 
         String content = handler.toString();
         assertContains("Text that I added to the master slide", content);
+
+        //now test turning off master content
+        OfficeParserConfig config = new OfficeParserConfig();
+        config.setIncludeSlideMasterContent(false);
+        ParseContext context = new ParseContext();
+        context.set(OfficeParserConfig.class, config);
+        content = getXML("testPPT_masterText2.pptx", context).xml;
+        assertNotContained("Text that I added", content);
     }
 
     @Test
@@ -1519,7 +1562,7 @@ public class OOXMLParserTest extends TikaTest {
     public void testBatch() throws Exception {
         OfficeParserConfig officeParserConfig = new OfficeParserConfig();
         officeParserConfig.setUseSAXDocxExtractor(true);
-        long started = new Date().getTime();
+        long started = System.currentTimeMillis();
         int ex = 0;
         for (int i = 0; i < 100; i++) {
             for (File f : getResourceAsFile("/test-documents").listFiles()) {
@@ -1539,7 +1582,7 @@ public class OOXMLParserTest extends TikaTest {
                 }
             }
         }
-        System.out.println("elapsed: "+(new Date().getTime()-started) + " with " + ex + " exceptions");
+        System.out.println("elapsed: "+(System.currentTimeMillis()-started) + " with " + ex + " exceptions");
     }
 
     @Test
@@ -1827,6 +1870,12 @@ public class OOXMLParserTest extends TikaTest {
         assertEquals("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
                 xlsx.get(Metadata.CONTENT_TYPE));
     }
+
+    @Test(expected = org.apache.tika.exception.TikaException.class)
+    public void testCorruptedZip() throws Exception {
+        //TIKA_2446
+        getRecursiveMetadata("testZIP_corrupted_oom.zip");
+    }
 }
 
 
