@@ -1247,16 +1247,24 @@ public class PDFParserTest extends TikaTest {
             context.set(Parser.class, new AutoDetectParser());
             //make sure everything works with regular xml _and_ with recursive
             XMLResult xmlResult = getXML("testPDFEmbeddingAndEmbedded.docx", context);
-            assertContains("pdf_haystack", xmlResult.xml);
+            //can get dehaystack depending on version of tesseract and/or preprocessing
+            if (xmlResult.xml.contains("pdf_haystack") || xmlResult.xml.contains("dehaystack")) {
+                //great
+            } else {
+                fail("couldn't find pdf_haystack or its variants");
+            }
             assertContains("Haystack", xmlResult.xml);
             assertContains("Needle", xmlResult.xml);
             if (! strategy.equals(PDFParserConfig.OCR_STRATEGY.NO_OCR)) {
                 // Tesseract may see the t in haystack as a ! some times...
-                String div = "<div class=\"ocr\">pdf_hays";
-                if (xmlResult.xml.contains(div+"!ack")) {
-                   assertContains(div+"!ack", xmlResult.xml);
+                //or it might see dehayslack...
+                //TODO: figure out how to make this test less hacky
+                String div = "<div class=\"ocr\">";
+                if (xmlResult.xml.contains(div+"pdf_hays!ack")) {
+                } else if (xmlResult.xml.contains(div+"pdf_haystack")) {
+                } else if (xmlResult.xml.contains(div+"dehayslack")) {
                 } else {
-                   assertContains(div+"tack", xmlResult.xml);
+                    fail("couldn't find acceptable variants of haystack");
                 }
             } else {
                 assertNotContained("<div class=\"ocr\">pdf_haystack", xmlResult.xml);
@@ -1378,6 +1386,20 @@ public class PDFParserTest extends TikaTest {
                 .metadata.get(TikaCoreProperties.LANGUAGE));
     }
 
+    @Test
+    public void testAngles() throws Exception {
+        PDFParserConfig pdfParserConfig = new PDFParserConfig();
+        pdfParserConfig.setDetectAngles(true);
+        ParseContext parseContext = new ParseContext();
+        parseContext.set(PDFParserConfig.class, pdfParserConfig);
+        String xml = getXML("testPDF_angles.pdf", parseContext).xml;
+        //make sure there is only one page!
+        assertContainsCount("<div class=\"page\">", xml, 1);
+        assertContains("IN-DEMAND", xml);
+        assertContains("natural underground", xml);
+        assertContains("transport mined materials", xml);
+    }
+
     /**
      * Simple class to count end of document events.  If functionality is useful,
      * move to org.apache.tika in src/test
