@@ -17,8 +17,39 @@
 
 package org.apache.tika.server.resource;
 
-import javax.mail.internet.ContentDisposition;
-import javax.mail.internet.ParseException;
+import org.apache.commons.lang3.StringUtils;
+import org.apache.cxf.attachment.ContentDisposition;
+import org.apache.cxf.jaxrs.ext.multipart.Attachment;
+import org.apache.poi.ooxml.extractor.ExtractorFactory;
+import org.apache.tika.Tika;
+import org.apache.tika.config.TikaConfig;
+import org.apache.tika.detect.Detector;
+import org.apache.tika.exception.EncryptedDocumentException;
+import org.apache.tika.metadata.Metadata;
+import org.apache.tika.metadata.TikaCoreProperties;
+import org.apache.tika.metadata.TikaMetadataKeys;
+import org.apache.tika.mime.MediaType;
+import org.apache.tika.parser.AutoDetectParser;
+import org.apache.tika.parser.DigestingParser;
+import org.apache.tika.parser.ParseContext;
+import org.apache.tika.parser.Parser;
+import org.apache.tika.parser.ParserDecorator;
+import org.apache.tika.parser.PasswordProvider;
+import org.apache.tika.parser.html.BoilerpipeContentHandler;
+import org.apache.tika.parser.html.HtmlParser;
+import org.apache.tika.parser.ocr.TesseractOCRConfig;
+import org.apache.tika.parser.pdf.PDFParserConfig;
+import org.apache.tika.sax.BodyContentHandler;
+import org.apache.tika.sax.ExpandedTitleContentHandler;
+import org.apache.tika.sax.RichTextContentHandler;
+import org.apache.tika.server.InputStreamFactory;
+import org.apache.tika.server.ServerStatus;
+import org.apache.tika.server.TikaServerParseException;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.xml.sax.ContentHandler;
+import org.xml.sax.SAXException;
+
 import javax.ws.rs.Consumes;
 import javax.ws.rs.GET;
 import javax.ws.rs.POST;
@@ -37,57 +68,42 @@ import javax.xml.transform.TransformerConfigurationException;
 import javax.xml.transform.sax.SAXTransformerFactory;
 import javax.xml.transform.sax.TransformerHandler;
 import javax.xml.transform.stream.StreamResult;
-
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.Writer;
 import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
-import org.apache.commons.lang.StringUtils;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.cxf.jaxrs.ext.multipart.Attachment;
-import org.apache.poi.extractor.ExtractorFactory;
-import org.apache.tika.config.TikaConfig;
-import org.apache.tika.detect.Detector;
-import org.apache.tika.exception.EncryptedDocumentException;
-import org.apache.tika.io.IOUtils;
-import org.apache.tika.io.TikaInputStream;
-import org.apache.tika.metadata.Metadata;
-import org.apache.tika.metadata.TikaMetadataKeys;
-import org.apache.tika.mime.MediaType;
-import org.apache.tika.parser.AutoDetectParser;
-import org.apache.tika.parser.ParseContext;
-import org.apache.tika.parser.Parser;
-import org.apache.tika.parser.PasswordProvider;
-import org.apache.tika.parser.html.HtmlParser;
-import org.apache.tika.parser.ocr.TesseractOCRConfig;
-import org.apache.tika.parser.pdf.PDFParserConfig;
-import org.apache.tika.sax.BodyContentHandler;
-import org.apache.tika.sax.ExpandedTitleContentHandler;
-import org.apache.tika.server.RichTextContentHandler;
-import org.apache.tika.server.TikaServerParseException;
-import org.xml.sax.ContentHandler;
-import org.xml.sax.SAXException;
+import static java.nio.charset.StandardCharsets.UTF_8;
 
 @Path("/tika")
 public class TikaResource {
-    public static final String GREETING = "This is Tika Server. Please PUT\n";
+
+    private static Pattern ALLOWABLE_HEADER_CHARS = Pattern.compile("(?i)^[-/_+\\.A-Z0-9 ]+$");
+
+    public static final String GREETING = "This is Tika Server (" + new Tika().toString() + "). Please PUT\n";
     public static final String X_TIKA_OCR_HEADER_PREFIX = "X-Tika-OCR";
     public static final String X_TIKA_PDF_HEADER_PREFIX = "X-Tika-PDF";
 
+    private static final Logger LOG = LoggerFactory.getLogger(TikaResource.class);
 
-    private final Log logger = LogFactory.getLog(TikaResource.class);
-
-    private TikaConfig tikaConfig;
-
-    public TikaResource(TikaConfig tikaConfig) {
-        this.tikaConfig = tikaConfig;
+    private static TikaConfig tikaConfig;
+    private static DigestingParser.Digester digester = null;
+    private static InputStreamFactory inputStreamFactory = null;
+    private static ServerStatus SERVER_STATUS = null;
+    public static void init(TikaConfig config, DigestingParser.Digester digestr,
+                            InputStreamFactory iSF, ServerStatus serverStatus) {
+        tikaConfig = config;
+        digester = digestr;
+        inputStreamFactory = iSF;
+        SERVER_STATUS = serverStatus;
     }
 
     static {
@@ -95,14 +111,14 @@ public class TikaResource {
     }
 
     @SuppressWarnings("serial")
-    public static AutoDetectParser createParser(TikaConfig tikaConfig) {
-        final AutoDetectParser parser = new AutoDetectParser(tikaConfig);
+    public static Parser createParser() {
+        final Parser parser = new AutoDetectParser(tikaConfig);
 
-        Map<MediaType, Parser> parsers = parser.getParsers();
+        Map<MediaType, Parser> parsers = ((AutoDetectParser)parser).getParsers();
         parsers.put(MediaType.APPLICATION_XML, new HtmlParser());
-        parser.setParsers(parsers);
+        ((AutoDetectParser)parser).setParsers(parsers);
 
-        parser.setFallback(new Parser() {
+        ((AutoDetectParser)parser).setFallback(new Parser() {
             public Set<MediaType> getSupportedTypes(ParseContext parseContext) {
                 return parser.getSupportedTypes(parseContext);
             }
@@ -111,26 +127,28 @@ public class TikaResource {
                 throw new WebApplicationException(Response.Status.UNSUPPORTED_MEDIA_TYPE);
             }
         });
-
+        if (digester != null) {
+            return new DigestingParser(parser, digester);
+        }
         return parser;
     }
 
+    public static TikaConfig getConfig() {
+        return tikaConfig;
+    }
+
     public static String detectFilename(MultivaluedMap<String, String> httpHeaders) {
 
         String disposition = httpHeaders.getFirst("Content-Disposition");
         if (disposition != null) {
-            try {
-                ContentDisposition c = new ContentDisposition(disposition);
+            ContentDisposition c = new ContentDisposition(disposition);
 
-                // only support "attachment" dispositions
-                if ("attachment".equals(c.getDisposition())) {
-                    String fn = c.getParameter("filename");
-                    if (fn != null) {
-                        return fn;
-                    }
+            // only support "attachment" dispositions
+            if ("attachment".equals(c.getType())) {
+                String fn = c.getParameter("filename");
+                if (fn != null) {
+                    return fn;
                 }
-            } catch (ParseException e) {
-                // not a valid content-disposition field
             }
         }
 
@@ -140,22 +158,39 @@ public class TikaResource {
 
     public static void fillParseContext(ParseContext parseContext, MultivaluedMap<String, String> httpHeaders,
                                         Parser embeddedParser) {
-        TesseractOCRConfig ocrConfig = new TesseractOCRConfig();
-        PDFParserConfig pdfParserConfig = new PDFParserConfig();
+        //lazily initialize configs
+        //if a header is submitted, any params set in --tika-config tika-config.xml
+        //upon server startup will be ignored.
+        TesseractOCRConfig ocrConfig = null;
+        PDFParserConfig pdfParserConfig = null;
         for (String key : httpHeaders.keySet()) {
             if (StringUtils.startsWith(key, X_TIKA_OCR_HEADER_PREFIX)) {
+                ocrConfig = (ocrConfig == null) ? new TesseractOCRConfig() : ocrConfig;
                 processHeaderConfig(httpHeaders, ocrConfig, key, X_TIKA_OCR_HEADER_PREFIX);
             } else if (StringUtils.startsWith(key, X_TIKA_PDF_HEADER_PREFIX)) {
+                pdfParserConfig = (pdfParserConfig == null) ? new PDFParserConfig() : pdfParserConfig;
                 processHeaderConfig(httpHeaders, pdfParserConfig, key, X_TIKA_PDF_HEADER_PREFIX);
             }
         }
-        parseContext.set(TesseractOCRConfig.class, ocrConfig);
-        parseContext.set(PDFParserConfig.class, pdfParserConfig);
+        if (ocrConfig != null) {
+            parseContext.set(TesseractOCRConfig.class, ocrConfig);
+        }
+        if (pdfParserConfig != null) {
+            parseContext.set(PDFParserConfig.class, pdfParserConfig);
+        }
         if (embeddedParser != null) {
             parseContext.set(Parser.class, embeddedParser);
         }
     }
 
+    public static InputStream getInputStream(InputStream is, HttpHeaders headers) {
+        try {
+            return inputStreamFactory.getInputSteam(is, headers);
+        } catch (IOException e) {
+            throw new TikaServerParseException(e);
+        }
+    }
+
     /**
      * Utility method to set a property on a class via reflection.
      *
@@ -166,27 +201,105 @@ public class TikaResource {
      * @throws WebApplicationException thrown when field cannot be found.
      */
     private static void processHeaderConfig(MultivaluedMap<String, String> httpHeaders, Object object, String key, String prefix) {
-        try {
-            String property = StringUtils.removeStart(key, prefix);
-            Field field = object.getClass().getDeclaredField(StringUtils.uncapitalize(property));
-            field.setAccessible(true);
-            if (field.getType() == String.class) {
-                field.set(object, httpHeaders.getFirst(key));
-            } else if (field.getType() == int.class) {
-                field.setInt(object, Integer.parseInt(httpHeaders.getFirst(key)));
-            } else if (field.getType() == double.class) {
-                field.setDouble(object, Double.parseDouble(httpHeaders.getFirst(key)));
-            } else if (field.getType() == boolean.class) {
-                field.setBoolean(object, Boolean.parseBoolean(httpHeaders.getFirst(key)));
+
+        try {String property = StringUtils.removeStart(key, prefix);
+            Field field = null;
+            try {
+                field = object.getClass().getDeclaredField(StringUtils.uncapitalize(property));
+            } catch (NoSuchFieldException e) {
+                //swallow
             }
+            String setter = property;
+            setter = "set"+setter.substring(0,1).toUpperCase(Locale.US)+setter.substring(1);
+            //default assume string class
+            //if there's a more specific type, e.g. double, int, boolean
+            //try that.
+            Class clazz = String.class;
+            if (field != null) {
+                if (field.getType() == int.class || field.getType() == Integer.class) {
+                    clazz = int.class;
+                } else if (field.getType() == double.class) {
+                    clazz = double.class;
+                } else if (field.getType() == Double.class) {
+                    clazz = Double.class;
+                } else if (field.getType() == float.class) {
+                    clazz = float.class;
+                } else if (field.getType() == Float.class) {
+                    clazz = Float.class;
+                } else if (field.getType() == boolean.class) {
+                    clazz = boolean.class;
+                } else if (field.getType() == Boolean.class) {
+                    clazz = Boolean.class;
+                }
+            }
+
+            Method m = tryToGetMethod(object, setter, clazz);
+            //if you couldn't find more specific setter, back off
+            //to string setter and try that.
+            if (m == null && clazz != String.class) {
+                m = tryToGetMethod(object, setter, String.class);
+            }
+
+            if (m != null) {
+                String val = httpHeaders.getFirst(key);
+                val = val.trim();
+                if (clazz == String.class) {
+                    checkTrustWorthy(setter, val);
+                    m.invoke(object, val);
+                } else if (clazz == int.class || clazz == Integer.class) {
+                    m.invoke(object, Integer.parseInt(val));
+                } else if (clazz == double.class || clazz == Double.class) {
+                    m.invoke(object, Double.parseDouble(val));
+                } else if (clazz == boolean.class || clazz == Boolean.class) {
+                    m.invoke(object, Boolean.parseBoolean(val));
+                } else if (clazz == float.class || clazz == Float.class) {
+                    m.invoke(object, Float.parseFloat(val));
+                } else {
+                    throw new IllegalArgumentException("setter must be String, int, float, double or boolean...for now");
+                }
+            } else {
+                throw new NoSuchMethodException("Couldn't find: "+setter);
+            }
+
         } catch (Throwable ex) {
             throw new WebApplicationException(String.format(Locale.ROOT,
                     "%s is an invalid %s header", key, X_TIKA_OCR_HEADER_PREFIX));
         }
     }
 
+    private static void checkTrustWorthy(String setter, String val) {
+        if (setter == null || val == null) {
+            throw new IllegalArgumentException("setter and val must not be null");
+        }
+        if (setter.toLowerCase(Locale.US).contains("trusted")) {
+            throw new IllegalArgumentException("Can't call a trusted method via tika-server headers");
+        }
+        Matcher m = ALLOWABLE_HEADER_CHARS.matcher(val);
+        if (! m.find()) {
+            throw new IllegalArgumentException("Header val: "+val +" contains illegal characters. " +
+                    "Must contain: TikaResource.ALLOWABLE_HEADER_CHARS");
+        }
+    }
+
+    /**
+     * Tries to get method. Silently swallows NoMethodException and returns
+     * <code>null</code> if not found.
+     * @param object
+     * @param method
+     * @param clazz
+     * @return
+     */
+    private static Method tryToGetMethod(Object object, String method, Class clazz) {
+        try {
+            return object.getClass().getMethod(method, clazz);
+        } catch (NoSuchMethodException e) {
+            //swallow
+        }
+        return null;
+    }
+
     @SuppressWarnings("serial")
-    public static void fillMetadata(AutoDetectParser parser, Metadata metadata, ParseContext context, MultivaluedMap<String, String> httpHeaders) {
+    public static void fillMetadata(Parser parser, Metadata metadata, ParseContext context, MultivaluedMap<String, String> httpHeaders) {
         String fileName = detectFilename(httpHeaders);
         if (fileName != null) {
             metadata.set(TikaMetadataKeys.RESOURCE_NAME_KEY, fileName);
@@ -206,14 +319,19 @@ public class TikaResource {
         if (mediaType != null) {
             metadata.add(org.apache.tika.metadata.HttpHeaders.CONTENT_TYPE, mediaType.toString());
 
-            final Detector detector = parser.getDetector();
+            final Detector detector = getDetector(parser);
 
-            parser.setDetector(new Detector() {
+            setDetector(parser, new Detector() {
                 public MediaType detect(InputStream inputStream, Metadata metadata) throws IOException {
                     String ct = metadata.get(org.apache.tika.metadata.HttpHeaders.CONTENT_TYPE);
-
+                    //make sure never to return null -- TIKA-1845
+                    MediaType type = null;
                     if (ct != null) {
-                        return MediaType.parse(ct);
+                        //this can return null if ct is not a valid mime type
+                        type = MediaType.parse(ct);
+                    }
+                    if (type != null) {
+                        return type;
                     } else {
                         return detector.detect(inputStream, metadata);
                     }
@@ -232,49 +350,92 @@ public class TikaResource {
         }
     }
 
-    public static void parse(Parser parser, Log logger, String path, InputStream inputStream,
+    public static void setDetector(Parser p, Detector detector) {
+        AutoDetectParser adp = getAutoDetectParser(p);
+        adp.setDetector(detector);
+    }
+
+    public static Detector getDetector(Parser p) {
+        AutoDetectParser adp = getAutoDetectParser(p);
+        return adp.getDetector();
+    }
+
+    private static AutoDetectParser getAutoDetectParser(Parser p) {
+        //bit stinky
+        if (p instanceof AutoDetectParser) {
+            return (AutoDetectParser)p;
+        } else if (p instanceof ParserDecorator) {
+            Parser wrapped = ((ParserDecorator)p).getWrappedParser();
+            if (wrapped instanceof AutoDetectParser) {
+                return (AutoDetectParser)wrapped;
+            }
+            throw new RuntimeException("Couldn't find AutoDetectParser within: "+wrapped.getClass());
+
+        }
+        throw new RuntimeException("Couldn't find AutoDetectParser within: "+p.getClass());
+
+    }
+
+    /**
+     * Use this to call a parser and unify exception handling.
+     * NOTE: This call to parse closes the InputStream. DO NOT surround
+     * the call in an auto-close block.
+     *
+     * @param parser parser to use
+     * @param logger logger to use
+     * @param path file path
+     * @param inputStream inputStream (which is closed by this call!)
+     * @param handler handler to use
+     * @param metadata metadata
+     * @param parseContext parse context
+     * @throws IOException wrapper for all exceptions
+     */
+    public static void parse(Parser parser, Logger logger, String path, InputStream inputStream,
                              ContentHandler handler, Metadata metadata, ParseContext parseContext) throws IOException {
+
+        checkIsOperating();
+        String fileName = metadata.get(Metadata.RESOURCE_NAME_KEY);
+        long taskId = SERVER_STATUS.start(ServerStatus.TASK.PARSE,
+                fileName);
         try {
             parser.parse(inputStream, handler, metadata, parseContext);
         } catch (SAXException e) {
             throw new TikaServerParseException(e);
         } catch (EncryptedDocumentException e) {
-            logger.warn(String.format(
-                    Locale.ROOT,
-                    "%s: Encrypted document",
-                    path
-            ), e);
+            logger.warn("{}: Encrypted document ({})", path, fileName, e);
             throw new TikaServerParseException(e);
         } catch (Exception e) {
-            logger.warn(String.format(
-                    Locale.ROOT,
-                    "%s: Text extraction failed",
-                    path
-            ), e);
+            logger.warn("{}: Text extraction failed ({})", path, fileName, e);
             throw new TikaServerParseException(e);
+        } catch (OutOfMemoryError e) {
+            logger.warn("{}: OOM ({})", path, fileName, e);
+            SERVER_STATUS.setStatus(ServerStatus.STATUS.ERROR);
+            throw e;
+        } finally {
+            SERVER_STATUS.complete(taskId);
+            inputStream.close();
         }
     }
 
-    public static void logRequest(Log logger, UriInfo info, Metadata metadata) {
+    public static void checkIsOperating() {
+        //check that server is not in shutdown mode
+        if (! SERVER_STATUS.isOperating()) {
+            throw new WebApplicationException(Response.Status.SERVICE_UNAVAILABLE);
+        }
+    }
+
+    public static void logRequest(Logger logger, UriInfo info, Metadata metadata) {
         if (metadata.get(org.apache.tika.metadata.HttpHeaders.CONTENT_TYPE) == null) {
-            logger.info(String.format(
-                    Locale.ROOT,
-                    "%s (autodetecting type)",
-                    info.getPath()
-            ));
+            logger.info("{} (autodetecting type)", info.getPath());
         } else {
-            logger.info(String.format(
-                    Locale.ROOT,
-                    "%s (%s)",
-                    info.getPath(),
-                    metadata.get(org.apache.tika.metadata.HttpHeaders.CONTENT_TYPE)
-            ));
+            logger.info("{} ({})", info.getPath(), metadata.get(org.apache.tika.metadata.HttpHeaders.CONTENT_TYPE));
         }
     }
 
     @GET
     @Produces("text/plain")
     public String getMessage() {
+        checkIsOperating();
         return GREETING;
     }
 
@@ -286,36 +447,70 @@ public class TikaResource {
         return produceText(att.getObject(InputStream.class), att.getHeaders(), info);
     }
 
+    //this is equivalent to text-main in tika-app
     @PUT
     @Consumes("*/*")
     @Produces("text/plain")
-    public StreamingOutput getText(final InputStream is, @Context HttpHeaders httpHeaders, @Context final UriInfo info) {
-        return produceText(is, httpHeaders.getRequestHeaders(), info);
+    @Path("main")
+    public StreamingOutput getTextMain(final InputStream is, @Context HttpHeaders httpHeaders, @Context final UriInfo info) {
+        return produceTextMain(is, httpHeaders.getRequestHeaders(), info);
     }
 
-    public StreamingOutput produceText(final InputStream is, MultivaluedMap<String, String> httpHeaders, final UriInfo info) {
-        final AutoDetectParser parser = createParser(tikaConfig);
+    //this is equivalent to text-main (Boilerpipe handler) in tika-app
+    @POST
+    @Consumes("multipart/form-data")
+    @Produces("text/plain")
+    @Path("form/main")
+    public StreamingOutput getTextMainFromMultipart(final Attachment att, @Context final UriInfo info) {
+        return produceTextMain(att.getObject(InputStream.class), att.getHeaders(), info);
+    }
+
+    public StreamingOutput produceTextMain(final InputStream is, MultivaluedMap<String, String> httpHeaders, final UriInfo info) {
+        final Parser parser = createParser();
         final Metadata metadata = new Metadata();
         final ParseContext context = new ParseContext();
 
         fillMetadata(parser, metadata, context, httpHeaders);
         fillParseContext(context, httpHeaders, parser);
 
-        logRequest(logger, info, metadata);
+        logRequest(LOG, info, metadata);
 
         return new StreamingOutput() {
             public void write(OutputStream outputStream) throws IOException, WebApplicationException {
-                Writer writer = new OutputStreamWriter(outputStream, IOUtils.UTF_8);
+                Writer writer = new OutputStreamWriter(outputStream, UTF_8);
+
+                ContentHandler handler = new BoilerpipeContentHandler(writer);
+
+                parse(parser, LOG, info.getPath(), is, handler, metadata, context);
+            }
+        };
+    }
+
+
+    @PUT
+    @Consumes("*/*")
+    @Produces("text/plain")
+    public StreamingOutput getText(final InputStream is, @Context HttpHeaders httpHeaders, @Context final UriInfo info) {
+        return produceText(getInputStream(is, httpHeaders), httpHeaders.getRequestHeaders(), info);
+    }
+
+    public StreamingOutput produceText(final InputStream is, MultivaluedMap<String, String> httpHeaders, final UriInfo info) {
+        final Parser parser = createParser();
+        final Metadata metadata = new Metadata();
+        final ParseContext context = new ParseContext();
+
+        fillMetadata(parser, metadata, context, httpHeaders);
+        fillParseContext(context, httpHeaders, parser);
+
+        logRequest(LOG, info, metadata);
+
+        return new StreamingOutput() {
+            public void write(OutputStream outputStream) throws IOException, WebApplicationException {
+                Writer writer = new OutputStreamWriter(outputStream, UTF_8);
 
                 BodyContentHandler body = new BodyContentHandler(new RichTextContentHandler(writer));
 
-                TikaInputStream tis = TikaInputStream.get(is);
-
-                try {
-                    parse(parser, logger, info.getPath(), tis, body, metadata, context);
-                } finally {
-                    tis.close();
-                }
+                parse(parser, LOG, info.getPath(), is, body, metadata, context);
             }
         };
     }
@@ -332,7 +527,7 @@ public class TikaResource {
     @Consumes("*/*")
     @Produces("text/html")
     public StreamingOutput getHTML(final InputStream is, @Context HttpHeaders httpHeaders, @Context final UriInfo info) {
-        return produceOutput(is, httpHeaders.getRequestHeaders(), info, "html");
+        return produceOutput(getInputStream(is, httpHeaders), httpHeaders.getRequestHeaders(), info, "html");
     }
 
     @POST
@@ -347,12 +542,12 @@ public class TikaResource {
     @Consumes("*/*")
     @Produces("text/xml")
     public StreamingOutput getXML(final InputStream is, @Context HttpHeaders httpHeaders, @Context final UriInfo info) {
-        return produceOutput(is, httpHeaders.getRequestHeaders(), info, "xml");
+        return produceOutput(getInputStream(is, httpHeaders), httpHeaders.getRequestHeaders(), info, "xml");
     }
 
     private StreamingOutput produceOutput(final InputStream is, final MultivaluedMap<String, String> httpHeaders,
                                           final UriInfo info, final String format) {
-        final AutoDetectParser parser = createParser(tikaConfig);
+        final Parser parser = createParser();
         final Metadata metadata = new Metadata();
         final ParseContext context = new ParseContext();
 
@@ -360,12 +555,12 @@ public class TikaResource {
         fillParseContext(context, httpHeaders, parser);
 
 
-        logRequest(logger, info, metadata);
+        logRequest(LOG, info, metadata);
 
         return new StreamingOutput() {
             public void write(OutputStream outputStream)
                     throws IOException, WebApplicationException {
-                Writer writer = new OutputStreamWriter(outputStream, IOUtils.UTF_8);
+                Writer writer = new OutputStreamWriter(outputStream, UTF_8);
                 ContentHandler content;
 
                 try {
@@ -373,20 +568,14 @@ public class TikaResource {
                     TransformerHandler handler = factory.newTransformerHandler();
                     handler.getTransformer().setOutputProperty(OutputKeys.METHOD, format);
                     handler.getTransformer().setOutputProperty(OutputKeys.INDENT, "yes");
-                    handler.getTransformer().setOutputProperty(OutputKeys.ENCODING, IOUtils.UTF_8.name());
+                    handler.getTransformer().setOutputProperty(OutputKeys.ENCODING, UTF_8.name());
                     handler.setResult(new StreamResult(writer));
                     content = new ExpandedTitleContentHandler(handler);
                 } catch (TransformerConfigurationException e) {
                     throw new WebApplicationException(e);
                 }
 
-                TikaInputStream tis = TikaInputStream.get(is);
-
-                try {
-                    parse(parser, logger, info.getPath(), tis, content, metadata, context);
-                } finally {
-                    tis.close();
-                }
+                parse(parser, LOG, info.getPath(), is, content, metadata, context);
             }
         };
     }
